"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.computePubkey = computePubkey;
exports.createECDHSecret = createECDHSecret;
exports.default = void 0;
exports.privKeyTweakAdd = privKeyTweakAdd;
exports.privKeyTweakMul = privKeyTweakMul;
exports.pubKeyTweakAdd = pubKeyTweakAdd;
exports.pubKeyTweakMul = pubKeyTweakMul;
exports.secKeyVerify = secKeyVerify;
exports.sign = sign;
exports.verify = verify;

var _reactNative = require("react-native");

var _utils = _interopRequireDefault(require("./utils"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const LINKING_ERROR = `The package 'react-native-secp256k1-urnm' doesn't seem to be linked. Make sure: \n\n` + _reactNative.Platform.select({
  ios: "- You have run 'pod install'\n",
  default: ''
}) + '- You rebuilt the app after installing the package\n' + '- You are not using Expo managed workflow\n';
const Secp256k1Urnm = _reactNative.NativeModules.Secp256k1Urnm ? _reactNative.NativeModules.Secp256k1Urnm : new Proxy({}, {
  get() {
    throw new Error(LINKING_ERROR);
  }

});

async function verify(data, sig, pub) {
  return await Secp256k1Urnm.verify(_utils.default.removeBase64Padding(data), _utils.default.removeBase64Padding(sig), _utils.default.removeBase64Padding(pub));
}

async function sign(data, priv) {
  return _utils.default.addBase64Padding(await Secp256k1Urnm.sign(_utils.default.removeBase64Padding(data), _utils.default.removeBase64Padding(priv)));
}

async function secKeyVerify(priv) {
  return await Secp256k1Urnm.secKeyVerify(_utils.default.removeBase64Padding(priv));
}

async function computePubkey(priv, compressed) {
  return _utils.default.addBase64Padding(await Secp256k1Urnm.computePubkey(_utils.default.removeBase64Padding(priv), compressed));
}

async function privKeyTweakAdd(priv, tweak) {
  return _utils.default.addBase64Padding(await Secp256k1Urnm.privKeyTweakAdd(_utils.default.removeBase64Padding(priv), _utils.default.removeBase64Padding(tweak)));
}

async function privKeyTweakMul(priv, tweak) {
  return _utils.default.addBase64Padding(await Secp256k1Urnm.privKeyTweakMul(_utils.default.removeBase64Padding(priv), _utils.default.removeBase64Padding(tweak)));
}

async function pubKeyTweakMul(pub, tweak) {
  return _utils.default.addBase64Padding(await Secp256k1Urnm.pubKeyTweakMul(_utils.default.removeBase64Padding(pub), _utils.default.removeBase64Padding(tweak)));
}

async function pubKeyTweakAdd(pub, tweak) {
  return _utils.default.addBase64Padding(await Secp256k1Urnm.pubKeyTweakAdd(_utils.default.removeBase64Padding(pub), _utils.default.removeBase64Padding(tweak)));
}

async function createECDHSecret(priv, pub) {
  return _utils.default.addBase64Padding(await Secp256k1Urnm.createECDHSecret(_utils.default.removeBase64Padding(priv), _utils.default.removeBase64Padding(pub)));
}

var _default = {
  verify,
  sign,
  secKeyVerify,
  computePubkey,
  privKeyTweakAdd,
  privKeyTweakMul,
  pubKeyTweakMul,
  pubKeyTweakAdd,
  createECDHSecret
};
exports.default = _default;
//# sourceMappingURL=secp256k1base64.js.map