export declare function verify(sig: Uint8Array, msg32: Uint8Array, pubkey: Uint8Array): Promise<boolean>;
export declare function sign(msg32: Uint8Array, seckey: Uint8Array): Promise<Uint8Array>;
export declare function privateKeyVerify(seckey: Uint8Array): Promise<boolean>;
export declare function publicKeyCreate(seckey: Uint8Array, compressed?: boolean): Promise<Uint8Array>;
export declare function createECDHSecret(seckey: Uint8Array, pubkey: Uint8Array): Promise<Uint8Array>;
export declare function privateKeyTweakMul(seckey: Uint8Array, tweak: Uint8Array): Promise<Uint8Array>;
export declare function privateKeyTweakAdd(seckey: Uint8Array, tweak: Uint8Array): Promise<Uint8Array>;
export declare function pubKeyTweakAdd(pubkey: Uint8Array, tweak: Uint8Array): Promise<Uint8Array>;
export declare function pubKeyTweakMul(pubkey: Uint8Array, tweak: Uint8Array): Promise<Uint8Array>;
export declare const ext: {
    generateKey: typeof import("./ext").generateKey;
    encryptECDH: typeof import("./ext").encryptECDH;
    decryptECDH: typeof import("./ext").decryptECDH;
};
export declare const utils: {
    decodeHex: typeof import("@stablelib/hex").decode;
    encodeHex: typeof import("@stablelib/hex").encode;
    decodeBase64: typeof import("@stablelib/base64").decode;
    encodeBase64: typeof import("@stablelib/base64").encode;
    decodeBase64WithoutPadding: typeof import("./utils").decodeBase64WithoutPadding;
    encodeBase64WithoutPadding: typeof import("./utils").encodeBase64WithoutPadding;
    removeBase64Padding: (s: string) => string;
    addBase64Padding: (data: string) => string;
};
export declare const base64: {
    verify: typeof import("./secp256k1base64").verify;
    sign: typeof import("./secp256k1base64").sign;
    secKeyVerify: typeof import("./secp256k1base64").secKeyVerify;
    computePubkey: typeof import("./secp256k1base64").computePubkey;
    privKeyTweakAdd: typeof import("./secp256k1base64").privKeyTweakAdd;
    privKeyTweakMul: typeof import("./secp256k1base64").privKeyTweakMul;
    pubKeyTweakMul: typeof import("./secp256k1base64").pubKeyTweakMul;
    pubKeyTweakAdd: typeof import("./secp256k1base64").pubKeyTweakAdd;
    createECDHSecret: typeof import("./secp256k1base64").createECDHSecret;
};
declare const _default: {
    verify: typeof verify;
    sign: typeof sign;
    privateKeyVerify: typeof privateKeyVerify;
    publicKeyCreate: typeof publicKeyCreate;
    privateKeyTweakAdd: typeof privateKeyTweakAdd;
    privateKeyTweakMul: typeof privateKeyTweakMul;
    pubKeyTweakMul: typeof pubKeyTweakMul;
    pubKeyTweakAdd: typeof pubKeyTweakAdd;
    createECDHSecret: typeof createECDHSecret;
    ext: {
        generateKey: typeof import("./ext").generateKey;
        encryptECDH: typeof import("./ext").encryptECDH;
        decryptECDH: typeof import("./ext").decryptECDH;
    };
    utils: {
        decodeHex: typeof import("@stablelib/hex").decode;
        encodeHex: typeof import("@stablelib/hex").encode;
        decodeBase64: typeof import("@stablelib/base64").decode;
        encodeBase64: typeof import("@stablelib/base64").encode;
        decodeBase64WithoutPadding: typeof import("./utils").decodeBase64WithoutPadding;
        encodeBase64WithoutPadding: typeof import("./utils").encodeBase64WithoutPadding;
        removeBase64Padding: (s: string) => string;
        addBase64Padding: (data: string) => string;
    };
    base64: {
        verify: typeof import("./secp256k1base64").verify;
        sign: typeof import("./secp256k1base64").sign;
        secKeyVerify: typeof import("./secp256k1base64").secKeyVerify;
        computePubkey: typeof import("./secp256k1base64").computePubkey;
        privKeyTweakAdd: typeof import("./secp256k1base64").privKeyTweakAdd;
        privKeyTweakMul: typeof import("./secp256k1base64").privKeyTweakMul;
        pubKeyTweakMul: typeof import("./secp256k1base64").pubKeyTweakMul;
        pubKeyTweakAdd: typeof import("./secp256k1base64").pubKeyTweakAdd;
        createECDHSecret: typeof import("./secp256k1base64").createECDHSecret;
    };
};
export default _default;
